@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2016-2018 Pytroll
+# Copyright (c) 2016-2019 Pytroll
 
 # Author(s):
 
@@ -71,7 +71,62 @@ class BandFrequencyOutOfRange(ValueError):
     pass
 
 
-class Rayleigh(object):
+class RayleighConfigBaseClass(object):
+    """A base class for the Atmospheric correction, handling the configuration and LUT download"""
+
+    def __init__(self, aerosol_type, atm_type='us-standard'):
+        """Initialize class and determine LUT to use."""
+        options = get_config()
+        self.do_download = False
+        self._lutfiles_version_uptodate = False
+
+        if 'download_from_internet' in options and options['download_from_internet']:
+            self.do_download = True
+
+        if atm_type not in ATMOSPHERES:
+            raise AttributeError('Atmosphere type not supported! ' +
+                                 'Need to be one of {}'.format(str(ATMOSPHERES)))
+
+        self._aerosol_type = aerosol_type
+        if aerosol_type not in AEROSOL_TYPES:
+            raise AttributeError('Aerosol type not supported! ' +
+                                 'Need to be one of {0}'.format(str(AEROSOL_TYPES)))
+
+        if atm_type not in ATMOSPHERES.keys():
+            LOG.error("Atmosphere type %s not supported", atm_type)
+
+        LOG.info("Atmosphere chosen: %s", atm_type)
+
+        if (self._aerosol_type in ATM_CORRECTION_LUT_VERSION and
+                self._get_lutfiles_version() == ATM_CORRECTION_LUT_VERSION[self._aerosol_type]['version']):
+            self.lutfiles_version_uptodate = True
+
+    def _get_lutfiles_version(self):
+        """Check the version of the atm correction luts from the version file in the
+           specific aerosol correction directory
+
+        """
+        basedir = RAYLEIGH_LUT_DIRS[self._aerosol_type]
+        lutfiles_version_path = os.path.join(basedir,
+                                             ATM_CORRECTION_LUT_VERSION[self._aerosol_type]['filename'])
+
+        if not os.path.exists(lutfiles_version_path):
+            return "v0.0.0"
+
+        with open(lutfiles_version_path, 'r') as fpt:
+            # Get the version from the file
+            return fpt.readline().strip()
+
+    @property
+    def lutfiles_version_uptodate(self):
+        return self._lutfiles_version_uptodate
+
+    @lutfiles_version_uptodate.setter
+    def lutfiles_version_uptodate(self, value):
+        self._lutfiles_version_uptodate = value
+
+
+class Rayleigh(RayleighConfigBaseClass):
     """Container for the atmospheric correction of satellite imager bands.
 
     This class removes background contributions of Rayleigh scattering of
@@ -81,31 +136,15 @@ class Rayleigh(object):
 
     def __init__(self, platform_name, sensor, **kwargs):
         """Initialize class and determine LUT to use."""
+
+        atm_type = kwargs.get('atmosphere', 'us-standard')
+        aerosol_type = kwargs.get('aerosol_type', 'marine_clean_aerosol')
+
+        super(Rayleigh, self).__init__(aerosol_type, atm_type)
+
         self.platform_name = platform_name
         self.sensor = sensor
         self.coeff_filename = None
-        options = get_config()
-        self.do_download = False
-        self._lutfiles_version_uptodate = False
-
-        atm_type = kwargs.get('atmosphere', 'us-standard')
-        if atm_type not in ATMOSPHERES:
-            raise AttributeError('Atmosphere type not supported! ' +
-                                 'Need to be one of {}'.format(str(ATMOSPHERES)))
-
-        aerosol_type = kwargs.get('aerosol_type', 'marine_clean_aerosol')
-        self._aerosol_type = aerosol_type
-
-        if aerosol_type not in AEROSOL_TYPES:
-            raise AttributeError('Aerosol type not supported! ' +
-                                 'Need to be one of {0}'.format(str(AEROSOL_TYPES)))
-
-        rayleigh_dir = RAYLEIGH_LUT_DIRS[aerosol_type]
-
-        if atm_type not in ATMOSPHERES.keys():
-            LOG.error("Atmosphere type %s not supported", atm_type)
-
-        LOG.info("Atmosphere chosen: %s", atm_type)
 
         # Try fix instrument naming
         instr = INSTRUMENTS.get(platform_name, sensor)
@@ -116,12 +155,7 @@ class Rayleigh(object):
 
         self.sensor = sensor.replace('/', '')
 
-        if 'download_from_internet' in options and options['download_from_internet']:
-            self.do_download = True
-
-        if (self._aerosol_type in ATM_CORRECTION_LUT_VERSION and
-                self._get_lutfiles_version() == ATM_CORRECTION_LUT_VERSION[self._aerosol_type]['version']):
-            self._lutfiles_version_uptodate = True
+        rayleigh_dir = RAYLEIGH_LUT_DIRS[aerosol_type]
 
         ext = atm_type.replace(' ', '_')
         lutname = "rayleigh_lut_{0}.h5".format(ext)
@@ -144,22 +178,6 @@ class Rayleigh(object):
         self._satz_sec_coord = None
         self._sunz_sec_coord = None
 
-    def _get_lutfiles_version(self):
-        """Check the version of the atm correction luts from the version file in the
-           specific aerosol correction directory
-
-        """
-        basedir = RAYLEIGH_LUT_DIRS[self._aerosol_type]
-        lutfiles_version_path = os.path.join(basedir,
-                                             ATM_CORRECTION_LUT_VERSION[self._aerosol_type]['filename'])
-
-        if not os.path.exists(lutfiles_version_path):
-            return "v0.0.0"
-
-        with open(lutfiles_version_path, 'r') as fpt:
-            # Get the version from the file
-            return fpt.readline().strip()
-
     def get_effective_wavelength(self, bandname):
         """Get the effective wavelength with Rayleigh scattering in mind"""
         try:
@@ -171,7 +189,10 @@ class Rayleigh(object):
                 LOG.warning(
                     "Effective wavelength is set to the requested band wavelength = %f", bandname)
                 return bandname
-            return None
+
+            msg = ("Can't get effective wavelength for band %s on platform %s and sensor %s" %
+                   (str(bandname), self.platform_name, self.sensor))
+            raise KeyError(msg)
 
         if isinstance(bandname, str):
             bandname = BANDNAMES.get(self.sensor, BANDNAMES['generic']).get(bandname, bandname)
@@ -204,6 +225,12 @@ class Rayleigh(object):
         return self._rayl, self._wvl_coord, self._azid_coord,\
             self._satz_sec_coord, self._sunz_sec_coord
 
+    @staticmethod
+    def _do_interp(minterp, sunzsec, azidiff, satzsec):
+        interp_points2 = np.vstack((sunzsec.ravel(), 180 - azidiff.ravel(), satzsec.ravel()))
+        res = minterp(interp_points2)
+        return res.reshape(sunzsec.shape)
+
     def get_reflectance(self, sun_zenith, sat_zenith, azidiff, bandname, redband=None):
         """Get the reflectance from the three sun-sat angles"""
         # Get wavelength in nm for band:
@@ -214,15 +241,9 @@ class Rayleigh(object):
             wvl = bandname * 1000.0
         else:
             wvl = self.get_effective_wavelength(bandname)
-            if wvl is None:
-                LOG.error("Can't get effective wavelength for band %s on platform %s and sensor %s",
-                          str(bandname), self.platform_name, self.sensor)
-                return None
-            else:
-                wvl = wvl * 1000.0
+            wvl = wvl * 1000.0
 
-        rayl, wvl_coord, azid_coord, satz_sec_coord, sunz_sec_coord = \
-            self.get_reflectance_lut()
+        rayl, wvl_coord, azid_coord, satz_sec_coord, sunz_sec_coord = self.get_reflectance_lut()
 
         # force dask arrays
         compute = False
@@ -249,8 +270,7 @@ class Rayleigh(object):
             LOG.info(
                 "Set the rayleigh/aerosol reflectance contribution to zero!")
             if HAVE_DASK:
-                chunks = sun_zenith.chunks if redband is None \
-                    else redband.chunks
+                chunks = sun_zenith.chunks if redband is None else redband.chunks
                 res = zeros(shape, chunks=chunks)
                 return res.compute() if compute else res
             else:
@@ -277,19 +297,11 @@ class Rayleigh(object):
         minterp = MultilinearInterpolator(smin, smax, orders)
         minterp.set_values(f_3d_grid)
 
-        def _do_interp(minterp, sunzsec, azidiff, satzsec):
-            interp_points2 = np.vstack((sunzsec.ravel(),
-                                        180 - azidiff.ravel(),
-                                        satzsec.ravel()))
-            res = minterp(interp_points2)
-            return res.reshape(sunzsec.shape)
-
         if HAVE_DASK:
-            ipn = map_blocks(_do_interp, minterp, sunzsec, azidiff,
-                             satzsec, dtype=raylwvl.dtype,
-                             chunks=azidiff.chunks)
+            ipn = map_blocks(self._do_interp, minterp, sunzsec, azidiff,
+                             satzsec, dtype=raylwvl.dtype, chunks=azidiff.chunks)
         else:
-            ipn = _do_interp(minterp, sunzsec, azidiff, satzsec)
+            ipn = self._do_interp(minterp, sunzsec, azidiff, satzsec)
 
         LOG.debug("Time - Interpolation: {0:f}".format(time.time() - tic))
 
@@ -337,3 +349,22 @@ def get_reflectance_lut(filename):
         h5f.close()
 
     return tab, wvl, azidiff, satellite_zenith_secant, sun_zenith_secant
+
+
+def check_and_download(**kwargs):
+    """Do a check for the version of the atmospheric correction LUTs and attempt
+       downloading only if needed
+
+    """
+
+    aerosol_types = kwargs.get('aerosol_types', AEROSOL_TYPES)
+    dry_run = kwargs.get('dry_run', False)
+
+    for aerosol_type in aerosol_types:
+        atmcorr = RayleighConfigBaseClass(aerosol_type)
+        if atmcorr.lutfiles_version_uptodate:
+            LOG.info("Atm correction LUTs, for aerosol distribution %s, already the latest!",
+                     aerosol_type)
+        else:
+            # Download
+            download_luts(aerosol_type=aerosol_type, dry_run=dry_run)
